<?php
/**
 * Plugin Name: Easify WooCommerce Connector
 * Plugin URI: http://www.woothemes.com/woocommerce/
 * Description: Connects Easify Business Management, EPOS (Electronic Point of Sale) and invoicing software to your WooCommerce enabled WordPress website. Allowing you to keep your online and offline shop's orders and stock synchronised.
 * Version: 1.0
 * Author: John Blakeman
 * Author URI: http://www.easify.co.uk/
 * Requires at least: 4.0
 * Tested up to: 4.0
 */

// exit if accessed directly
if( !defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Easify EASU WooCommerce Connector Class
 * This class hooks into WooCommerce waiting for a orders to be raised.
 * It then instantiates EasifySender that sends the order to Easify.
 */
class WooCommerce_Easify_EASU_Connector
{
	/**
	 * __construct function.
	 */
	public function __construct() {
		
		// check if WooCommerce is activated
		if ( !in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) )
			return;
		
		// start hooks
		$this->hooks();
		
	}
	
	/**
	 * hooks function.
	 *
	 * initialise hooks
	 */
	private function hooks() {
	
		// register to be notified by WooCommerce when an order is ready to be sent to Easify
		add_action( 'woocommerce_order_status_processing', array( $this, 'send_order_to_easify' ) );
		
	}
	
	/**
	 * send_order_to_easify function.
	 *
	 * send order to the Easify EASU web service
	 */
	public function send_order_to_easify( $order_id ) {
		
		// include the EasifySender class
		require_once ( dirname( __FILE__ ) . '/includes/easify_woocomm_send.php' );
		
		// EasifySender gathers all order data, then sends it to the Easify EASU web service
		$sender = new EasifySender( $order_id );
		
	}
}

$weec = new WooCommerce_Easify_EASU_Connector();

class WooCommerce_Easify_EASU_Settings
{
	/**
     * holds the Easify settings stored in the WordPress database
     */
    private $options;
	
	/**
	 * holds the order statuses imported from Easify 
	 */
	private $order_statuses;
	
	/**
	 * holds the order types imported from Easify 
	 */
	private $order_types;
	
	/**
	 * holds the customer types imported from Easify 
	 */	
	private $customer_types;
	
	/**
	 * holds the customer relationships imported from Easify 
	 */	
	private $customer_relationships;
	
	/**
	 * holds the payment methods imported from Easify 
	 */	
	private $payment_methods;
	
	/**
	 * holds the payment accounts imported from Easify 
	 */	
	private $payment_accounts;
	
	/**
	 * holds the payment terms imported from Easify 
	 */	
	private $payment_terms;
	
	/**
	 * maps the Easify web services methods to flags
	 */
	private $easify_web_methods = array(
		'order_statuses' => array(
			'cached_member_variable_name' => 'order_statuses', 
			'web_service' => 'Orders', 
			'method_call' => 'GetStatuses', 
			'result_set' => 'GetStatusesResult', 
			'result_method' => 'OrderStatuses'
		), 
		'order_types' => array(
			'cached_member_variable_name' => 'order_types', 
			'web_service' => 'Orders', 
			'method_call' => 'GetTypes', 
			'result_set' => 'GetTypesResult', 
			'result_method' => 'OrderTypes'
		), 
		'customer_types' => array(
			'cached_member_variable_name' => 'customer_types', 
			'web_service' => 'Customers', 
			'method_call' => 'GetTypes', 
			'result_set' => 'GetTypesResult', 
			'result_method' => 'CustomerTypes'
		), 
		'customer_relationships' => array(
			'cached_member_variable_name' => 'customer_relationships', 
			'web_service' => 'Customers', 
			'method_call' => 'GetRelationships', 
			'result_set' => 'GetRelationshipsResult', 
			'result_method' => 'Relationship'
		), 
		'payment_methods' => array(
			'cached_member_variable_name' => 'payment_methods', 
			'web_service' => 'Payments', 
			'method_call' => 'GetPaymentMethods', 
			'result_set' => 'GetPaymentMethodsResult', 
			'result_method' => 'PaymentMethods'
		), 
		'payment_accounts' => array(
			'cached_member_variable_name' => 'payment_accounts', 
			'web_service' => 'Payments', 
			'method_call' => 'GetPaymentAccounts', 
			'result_set' => 'GetPaymentAccountsResult', 
			'result_method' => 'PaymentAccounts'
		), 
		'payment_terms' => array(
			'cached_member_variable_name' => 'payment_terms', 
			'web_service' => 'Customers', 
			'method_call' => 'GetPaymentTerms', 
			'result_set' => 'GetPaymentTermsResult', 
			'result_method' => 'PaymentTerms'
		), 
	);
	
	/**
     * __construct function.
     */
    public function __construct() {
		
		// check if WooCommerce is activated
		if ( !in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) )
			return;
		
		// add Easify options menu into WordPress
        add_action( 'admin_menu', array( $this, 'add_easify_plugin_page' ) );
		
		// initialise Easify option pages
        add_action( 'admin_init', array( $this, 'init_easify_pages' ) );
		
		// initialise Easify style sheet
		add_action( 'admin_head', array( $this, 'init_easify_header' ) );
		
    }
	
	
	
	/**
     * add Easify options menu into WordPress
     */
    public function add_easify_plugin_page() {
	
        // adding Easify options page under "Settings"
        add_options_page(
            'Easify', 
            'Easify Settings', 
            'manage_options', 
            'easify_options_page', 
            array( $this, 'create_easify_options_page' ) 
        );
		
    }
	
	/**
     * initialise Easify option pages
     */
    public function init_easify_pages() { 
		
		$this->init_setup();
		$this->init_orders();
		$this->init_customers();
		$this->init_shipping();
		$this->init_payment();
		$this->init_logging();
		
    }
	
	/**
	 * checks if we have communications with the Easify server
	 */
	private function have_comms() {
		
		// HACK: set default_socket_timeout otherwise we cannot catch SoapFault exceptions...?!
		ini_set( "default_socket_timeout", 5 );
		
		// base names of each Easify web service
		$setup_options = array(
			"Products", 
			"Orders", 
			"Customers", 
			"Payments" 
		);
		
		try {
			
			// have we got the basics... private key and timeout values?
			if( !isset( $this->options[ 'easify_options_setup' ][ 'easify_private_key' ] ) || 
				empty( $this->options[ 'easify_options_setup' ][ 'easify_private_key' ] ) ) return false;
			
			if( !isset( $this->options[ 'easify_options_setup' ][ 'easify_web_service_timeout' ] ) || 
				empty( $this->options[ 'easify_options_setup' ][ 'easify_web_service_timeout' ] ) ) return false;
			
			// check each Easify web service is live
			foreach( $setup_options as $option ) {
				
				// open connection to Easify web service
				$client = new SoapClient(
					$this->options[ 'easify_options_setup' ][ 'easify_web_service_location' ] . 
						"Inbound/EasifyInbound" . $option . ".asmx?wsdl", 
					array(
						"connection_timeout" => $this->options[ 'easify_options_setup' ][ 'easify_web_service_timeout' ]
					)
				);
				
				// get response from Easify web service 
				$test_result = $client->HelloWorld( array( "PK" => $this->options[ 'easify_options_setup' ][ 'easify_private_key' ] ) );
				
				// check if this is the response we're looking for
				if( empty( $test_result->HelloWorldResult ) ) return false;
				
			}
			
		} catch( SoapFault $ex ) {
			
			return false;
			
		} catch( Exception $ex ) {
			
			return false;
			
		}
		
		return true;
		
	}
	
	/**
	 * check if an option has been saved to the WordPress database
	 */
	private function have_option( $option ) {
	
		if ( !isset( $this->options[ 'easify_options_setup' ][ $option ] ) || 
			empty( $this->options[ 'easify_options_setup' ][ $option ] ) ) return false;
		return true;
		
	}
	
	/**
	 * access Easify web methods based on given parameter, look up method access rules in $easify_web_methods
	 */
	private function get_easify_web_service_data( $web_service_method ) {
		
		// HACK: set default_socket_timeout otherwise we cannot catch SoapFault exceptions...?!
		ini_set("default_socket_timeout", 5);
		
		// don't get fresh value if we already have it in cache 
		if ( isset( $this->{ $this->easify_web_methods[ $web_service_method ][ 'cached_member_variable_name' ] } ) || 
			!empty( $this->{ $this->easify_web_methods[ $web_service_method ][ 'cached_member_variable_name' ] } ) ) return;
		
		try {
			
			// discover web service
			$web_service = new SoapClient(
				$this->options[ 'easify_options_setup' ][ 'easify_web_service_location' ] . 
					"Inbound/EasifyInbound" . $this->easify_web_methods[ $web_service_method ][ 'web_service' ] . ".asmx?wsdl", 
				array(
					"connection_timeout" => $this->options[ 'easify_options_setup' ][ 'easify_web_service_timeout' ]
				)
			);
			
			// call web method
			$result = $web_service->{ $this->easify_web_methods[ $web_service_method ][ 'method_call' ] }(
				array( "PK" => $this->options[ 'easify_options_setup' ][ 'easify_private_key' ] )
			);
			
			// get results
			$return_value = $result->{ 
				$this->easify_web_methods[ $web_service_method ][ 'result_set' ] }->{ 
				$this->easify_web_methods[ $web_service_method ][ 'result_method' ]
			};
			
			// return a value if we didn't get anything back
			if (empty($return_value)) return false;
			
			// cache returned value to reduce trips to the Easify web service
			$this->{ $this->easify_web_methods[ $web_service_method ][ 'cached_member_variable_name' ] } = $return_value;
			
		} catch (SoapFault $ex) {
			
			return false;
			
		} catch (Exception $ex) {
			
			return false;
			
		}
	}
	
	/**
	 * create Easify options page tabs
	 */
	private function create_easify_options_tabs( $current = 'setup' ) {
	
		// a list of the Easify option pages 
		$tabs = array( 
			'setup' => 'Setup', 
			'orders' => 'Orders', 
			'customers' => 'Customers', 
			'shipping' => 'Shipping', 
			'payment' => 'Payment', 
			'logging' => 'Logging'
		);
		
		// print Easify icon and start tab wrapper
		print '<div id="icon-themes" class="easify_icon"><a class="easify_link" target="_blank" href="http://www.easify.co.uk/">&nbsp;</a></div>';
		print '<h2 class="nav-tab-wrapper">';
		
		// check if we can communicate with Easify web services
		if ( !$this->have_comms() ) {
			
			// only display Setup tab if we do not have comms
			print "<a class='nav-tab nav-tab-active' href='?page=easify_options_page&tab=setup'>Setup</a>";
			
		} else {		
			
			// create each tab
			foreach( $tabs as $tab => $name ) {
				
				// if tab represents current page, highlight it
				$class = ( $tab == $current ) ? ' nav-tab-active' : '';
				print "<a class='nav-tab$class' href='?page=easify_options_page&tab=$tab'>$name</a>";
				
			}
			
		}
		
		// close wrapper
		print '</h2></div>';
	}
	
	/**
     * create Easify options page callback
     */
    public function create_easify_options_page() {
        
		// get current tab
		$tab = $_GET['tab'];
		
		// start page wrapper
		print '<div class="wrap">';
		print '<h2>Easify Settings</h2>';
		
		// get Easify options from WordPress database and store them
		$this->options[ 'easify_options_setup' ] = get_option( 'easify_options_setup' );
		$this->options[ 'easify_options_orders' ] = get_option( 'easify_options_orders' );
		$this->options[ 'easify_options_customers' ] = get_option( 'easify_options_customers' );
		$this->options[ 'easify_options_shipping' ] = get_option( 'easify_options_shipping' );
		$this->options[ 'easify_options_payment' ] = get_option( 'easify_options_payment' );
		$this->options[ 'easify_options_logging' ] = get_option( 'easify_options_logging' );
		
		// set default tab
		if ( !isset( $tab ) || empty( $tab ) )
			$tab = 'setup';
		
        // draw the tabs, highlighting current tab
		$this->create_easify_options_tabs( $tab );
		
		// start form wrapper 
		print '<form method="post" action="options.php">';
		
		// print hidden setting fields
		settings_fields( 'easify_options_' . $tab . '_group' );   
		
		// print current page 
		do_settings_sections( 'easify_options_page_' . $tab );
		
		// print submit / save button
		submit_button(); 
		
		// close page wrapper 
        print '</form></div>';
        
    }

	/**
	 * register Easify style sheet 
	 */
	public function init_easify_header() {
		
		print '<link rel="stylesheet" type="text/css" href="' . plugin_dir_url( __FILE__ ) . 'assets/css/easify.css" />';
		
		print '<script type="text/javascript" src="' . plugin_dir_url( __FILE__ ) . 'assets/js/jquery-1.7.0.min.js"></script>';
		print '<script type="text/javascript" src="' . plugin_dir_url( __FILE__ ) . 'assets/js/jquery.tooltipster.min.js"></script>';
		
		print "<script type='text/javascript'>
		$(document).ready(function() {
            $('.easify_tip').tooltipster({
                contentAsHTML: true
            });
        });
		</script>";
		
	}
	
	/**
	 * Tooltipster method
	 */
	private function tooltip( $help_id, $tip, $anchor = '' ) {
		
		// prints a help icon that links to Easify support articles and shows a tooltip when hovered over
		printf(
			'<a href="http://www.easify.co.uk/help/hs%s.aspx%s" target="_blank">
				<img class="easify_tip" title="%s" src="' . plugin_dir_url( __FILE__ ) . 'assets/images/help.png" 
					style="height: 16px; width: 16px;"/></a>', 
			$help_id, // Easify help article id
			!empty($anchor) ? '#' . $anchor : '', 
			$tip
		);
		
	}
	
	
	
	/**
	 * initialise Easify Setup page 
	 */
	private function init_setup() {
		
		// register Setup options group 
		register_setting(
            'easify_options_setup_group', // group
            'easify_options_setup', // name
            array( $this, 'sanitise' ) // sanitise method
        );
		
		// add Setup page 
		add_settings_section(
            'easify_options_section_setup', // id
            'Easify Setup Options', // title
            array( $this, 'settings_html_easify_setup_message' ), // callback
            'easify_options_page_setup' // page
        );
		
		// EASIFY_PK
		add_settings_field(
            'easify_private_key', // id
            'Easify\'s Private Key', // title
            array( $this, 'setting_html_easify_private_key' ), // callback
            'easify_options_page_setup', // page
            'easify_options_section_setup' // section
        );
		
		// EASIFY_WEBSERVICE_TIMEOUT
        add_settings_field(
            'easify_web_service_timeout',
            'Communication Timeout (seconds)',
            array( $this, 'setting_html_easify_web_service_timeout' ),
            'easify_options_page_setup',
            'easify_options_section_setup'
        );
		
		// EASIFY_WEB_SERVICES
		add_settings_field(
            'easify_web_service_location',
            'Easify Web Service Location (URL)',
            array( $this, 'setting_html_easify_web_service_location' ),
            'easify_options_page_setup',
            'easify_options_section_setup'
        );
		
		// WORDPRESS_WEB_SERVICES
		add_settings_field(
            'wordpress_web_service_location',
            'WordPress Web Service Location (URL)',
            array( $this, 'setting_html_wordpress_web_service_location' ),
            'easify_options_page_setup',
            'easify_options_section_setup'
        );
		
	}
	
	/** 
     * HTML to display for the Setup page 
     */
    public function settings_html_easify_setup_message() {
		
		// check if we can communicate with Easify web services
		if ( $this->have_comms() ) {
			
			$blurb = "Connection Established";
			?>
			<table class="easify_server_status">
				<tr>
					<td><?php print 'Easify Server Status' ?></td>
					<td><img class="easify_server_status" src="<?php print plugin_dir_url( __FILE__ ); ?>assets/images/success.png" 
						alt="<?php print $blurb; ?>" title="<?php print $blurb; ?>"/> - Easify server is online and accessible</td>
				</tr>
			</table>
			<?php 
			
		} else {
			
			$blurb = "Connection Failure";
			?>
			<table class="easify_server_status">
				<tr>
					<td><?php print 'Easify Server Status' ?></td>
					<td><img class="easify_server_status" src="<?php print plugin_dir_url( __FILE__ ); ?>assets/images/failure.png" 
						alt="<?php print $blurb; ?>" title="<?php print $blurb; ?>"/> - Check the Easify server is online and accessible</td>
				</tr>
			</table>
			<?php 
		}
    }
	
	/**
	 * HTML to display the EASIFY_PK config option
	 */
	public function setting_html_easify_private_key() {
		
		$this->tooltip('6020', 
			"The Easify Private Key needs to be set to the same secure password here and in Easify:&lt;br /&gt;&lt;br /&gt;
			&lt;code&gt;Easify > Tools > Options > General > Advanced Server Settings > Web Service Private Key&lt;/code&gt;\n", 
			'easifyprivatekey'
		);
		
		printf(
			'<input type="password" id="easify_private_key" name="easify_options_setup[easify_private_key]" style="width: 250px;" value="%s" />',
            isset( $this->options[ 'easify_options_setup' ][ 'easify_private_key' ] ) ? 
				esc_attr( $this->options[ 'easify_options_setup' ][ 'easify_private_key' ] ) : ''
		);
		
		// warn user if this option has not been set 
		if ( !$this->have_option( 'easify_private_key' ) ) {
			$blurb = "In Easify, go to 'Tools > Options > General > Advanced Server Settings > Web Service Private Key' to set and get the value we need to enter here...";
			print '<img style="padding-left: 5px;" src="' . plugin_dir_url( __FILE__ ) . 'assets/images/warning.png" alt="' . $blurb . '" title="' . $blurb . '"/>';
		}
		
	}
	
	/**
	 * HTML to display the EASIFY_WEBSERVICE_TIMEOUT config option
	 */
	public function setting_html_easify_web_service_timeout() {
		
		$this->tooltip('6020', 
			"Changing the communication timeout can help with stability", 
			'easifycommunicationtimeout'
		);
		
		printf(
			'<input type="text" id="easify_web_service_timeout" name="easify_options_setup[easify_web_service_timeout]" style="width: 40px;" value="%s" />',
            isset( $this->options[ 'easify_options_setup' ][ 'easify_web_service_timeout' ] ) ? 
				esc_attr( $this->options[ 'easify_options_setup' ][ 'easify_web_service_timeout' ] ) : ''
		);
		
		// warn user if this option has not been set 
		if ( !$this->have_option( 'easify_web_service_timeout' ) ) {
			$blurb = "Set this to how ever long you wish your web server to wait for your Easify server to respond. If you have a slow internet connection where you have installed Easify, increasing this value may help with stability...";
			print '<img style="padding-left: 5px;" src="' . plugin_dir_url( __FILE__ ) . 'assets/images/warning.png" alt="' . $blurb . '" title="' . $blurb . '"/>';
		}
	}
	
	/**
	 * HTML to display the EASIFY_WEB_SERVICE_LOCATION config option
	 */
	public function setting_html_easify_web_service_location() {
		
		$this->tooltip('6020', 
			"The location of your Easify web service.&lt;br /&gt;&lt;br /&gt;
			This is a combination of your external IP address and the company name you have set in Easify.&lt;br /&gt;&lt;br /&gt;
			&lt;b&gt;For example&lt;/b&gt;, if my external IP was 123.456.789 and my company name was Easify Electricals Ltd&lt;br /&gt;
			I would set the web service location to http://123.456.789/EasifyElectricalsLtd/&lt;br /&gt;&lt;br /&gt;
			&lt;span style=&quot;color:#ff0;&quot;&gt;&lt;b&gt;Please note&lt;/b&gt;&lt;/span&gt;, in most cases you will have to have allowed port 80 / 443 through any firewalls,&lt;br /&gt; 
			router NAT or router port forwarding security systems and made sure with your Internet&lt;br /&gt;
			Service Provider that you have a &lt;b&gt;Static IP&lt;/b&gt; before you can connect to your Easify server.", 
			'easifywebservicelocation'
		);
		
		printf(
			'<input type="text" id="easify_web_service_location" name="easify_options_setup[easify_web_service_location]" style="width: 350px;" value="%s" />',
            isset( $this->options['easify_options_setup']['easify_web_service_location'] ) ? esc_attr( $this->options['easify_options_setup']['easify_web_service_location']) : ''
		);
		
		// warn user if this option has not been set or fails to connect to Easify web service
		if ( !$this->have_comms() ) {
			$blurb = "Failed to connect";
			print '<img style="padding-left: 5px;" src="' . plugin_dir_url( __FILE__ ) . 'assets/images/warning.png" alt="' . $blurb . '" title="' . $blurb . '"/>';
		}
		
	}
	
	/**
	 * HTML to display WORDPRESS_WEB_SERVICE_LOCATION 
	 */
	public function setting_html_wordpress_web_service_location() {
		
		print(
			
			'<div style="float: left; display: inline-block;">' . 
				$this->tooltip('6020', 
					"Once you have installed the Easify WooCommerce Connector, Easify needs to know where it is.&lt;br /&gt;&lt;br /&gt;
					Copy and paste this URL into Easify in the following location:&lt;br /&gt;&lt;br /&gt;
					&lt;code&gt;Easify &gt; Tools &gt; Options &gt; General &gt; Advanced Server Settings &gt; URL of web services on your web server&lt;/code&gt;", 
					'wordpresswebservicelocation'
				)
			. '</div>' . 
			
			'<div class="wordpress_web_service_location">
				' . plugin_dir_url( __FILE__ ) . 'easify_web_service.php</div>'
			
		);
		
	}
	
	
	
	/**
	 * initialise Easify Orders page 
	 */
	private function init_orders() {
		
		// register Orders options group 
		register_setting(
            'easify_options_orders_group', // group
            'easify_options_orders', // name
            array( $this, 'sanitise' ) // sanitise method
        );
		
		// add Orders page 
		add_settings_section(
            'easify_options_section_orders', // id
            'Easify Orders Options', // title
            array( $this, 'settings_html_easify_orders_message' ), // callback
            'easify_options_page_orders' // page
        );
	
		// EASIFY_ORDER_STATUS_ID
		add_settings_field(
            'easify_order_status_id', // id 
            'Order Status', // title
            array( $this, 'setting_html_easify_order_status_id' ), //callback
            'easify_options_page_orders', // page
            'easify_options_section_orders' // section
        );
		
		// EASIFY_ORDER_TYPE_ID
		add_settings_field(
            'easify_order_type_id', 
            'Order Type', 
            array( $this, 'setting_html_easify_order_type_id' ), 
            'easify_options_page_orders', 
            'easify_options_section_orders' 
        );
		
		// EASIFY_ORDER_COMMENT
		add_settings_field(
            'easify_order_comment', 
            'Order Comment', 
            array( $this, 'setting_html_easify_order_comment' ), 
			'easify_options_page_orders', 
            'easify_options_section_orders' 
        );
		
	}
	
	/**
	 * HTML to display for the Orders page
	 */
	public function settings_html_easify_orders_message() {
		
		print 'These settings will be used when Orders are automatically raised in Easify...';
		
	}
	
	/**
	 * HTML to display the EASIFY_ORDER_STATUS_ID config option
	 */
	public function setting_html_easify_order_status_id() {
		
		$this->tooltip('6030', 
			"WooCommerce Orders will have this Order Status when they are raised in Easify", 
			'easifyorderstatus'
		);
		
		// select control start tag
		print '<select name="easify_options_orders[easify_order_status_id]" style="width: 150px;">';
		
		// get Easify order statuses if they've not been cached
		$this->get_easify_web_service_data( 'order_statuses' );
		
		// print each select option
		foreach( $this->order_statuses as $status ) {
			
			// print option, mark as selected if needed
			printf(
				'<option value="%s"%s>%s</option>', 
				$status->Id, 
				( $this->options[ 'easify_options_orders' ][ 'easify_order_status_id' ] == $status->Id ) || 
					( !isset($this->options[ 'easify_options_orders' ][ 'easify_order_status_id' ] ) && 
					$status->DefaultType == 'true' ) ? ' selected' : '', 
				$status->Description
			);
			
		}
		
		// select control close tag
		print '</select>';
		
	}
	
	/**
	 * HTML to display the EASIFY_ORDER_TYPE_ID config option
	 */
	public function setting_html_easify_order_type_id() {
		
		$this->tooltip('6030', 
			"WooCommerce Orders will have this Order Type when they are raised in Easify", 
			'easifyordertype'
		);
		
		// select control start tag
		print '<select name="easify_options_orders[easify_order_type_id]" style="width: 150px;">';
		
		// get Easify order types if they've not been cached
		$this->get_easify_web_service_data( 'order_types' );
		
		// print each select option
		foreach( $this->order_types as $type ) {
			
			// print option, mark as selected if needed
			printf(
				'<option value="%s"%s>%s</option>', 
				$type->Id, 
				($this->options[ 'easify_options_orders' ][ 'easify_order_type_id' ] == $type->Id ) || 
					( !isset( $this->options[ 'easify_options_orders' ][ 'easify_order_type_id' ] ) && 
					$type->Description == 'Internet' ) ? ' selected' : '', 
				$type->Description
			);
			
		}
		
		// select control close tag
		print '</select>';
		
	}
	
	/**
	 * HTML to display the EASIFY_ORDER_COMMENT config option
	 */
	public function setting_html_easify_order_comment() {
		
		$this->tooltip('6030', 
			"WooCommerce Orders will have this Order Comment followed by their WooCommerce order number when they are raised in Easify", 
			'easifyordercomment'
		);
		
		printf(
			'<input type="text" id="easify_order_comment" name="easify_options_orders[easify_order_comment]" style="width: 250px;" value="%s" />',
            isset( $this->options[ 'easify_options_orders' ][ 'easify_order_comment' ] ) ? 
				esc_attr( $this->options[ 'easify_options_orders' ][ 'easify_order_comment' ] ) : 'Internet Order'
		);
		
	}
	
	
	
	/**
	 * Initialise Easify Customers Settings
	 */
	private function init_customers() {
		
		// register Customers options group 
		register_setting(
            'easify_options_customers_group', // group
            'easify_options_customers', // name
            array( $this, 'sanitise' ) // sanitise method
        );
	
		// add Customers page 
		add_settings_section(
            'easify_options_section_customers', // id
            'Easify Orders Options', // title
            array( $this, 'settings_html_easify_customers_message' ), // callback
            'easify_options_page_customers' // page
        );
		
		// EASIFY_CUSTOMER_TYPE_ID
		add_settings_field(
            'easify_customer_type_id', // id
            'Customer Type', // display title
            array( $this, 'setting_html_easify_customer_type_id' ), // callback
			'easify_options_page_customers', // page
            'easify_options_section_customers' // section
        );
		
		// EASIFY_CUSTOMER_RELATIONSHIP_ID
		add_settings_field(
            'easify_customer_relationship_id', 
            'Customer Relationship', 
            array( $this, 'setting_html_easify_customer_relationship_id' ), 
			'easify_options_page_customers', 
            'easify_options_section_customers' 
        );
		
	}
	
	/**
	 * HTML to display for the Orders section
	 */
	public function settings_html_easify_customers_message() {
		
		print 'These settings will be used when Customers are automatically raised in Easify...';
		
	}
	
	/**
	 * HTML to display the EASIFY_CUSTOMER_TYPE_ID config option
	 */
	public function setting_html_easify_customer_type_id() {
		
		$this->tooltip('6040', 
			"WooCommerce Customers will have this Customer Type when they are raised in Easify", 
			'easifycustomertype'
		);
		
		// select control start tag
		print '<select name="easify_options_customers[easify_customer_type_id]" style="width: 150px;">';
		
		// get Easify customer types if they've not been cached
		$this->get_easify_web_service_data( 'customer_types' ); 
		
		// print each select option
		foreach( $this->customer_types as $type ) {
			
			// print option, mark as selected if needed
			printf (
				'<option value="%s"%s>%s</option>', 
				$type->Id, 
				( $this->options[ 'easify_options_customers' ][ 'easify_customer_type_id' ] == $type->Id ) || 
					( !isset( $this->options[ 'easify_options_customers' ][ 'easify_customer_type_id' ] ) && 
					$type->Description == 'Not Known' ) ? ' selected' : '', 
				$type->Description
			);
			
		}
		
		// select control close tag
		print '</select>';
		
	}
	
	/**
	 * HTML to display the EASIFY_CUSTOMER_RELATIONSHIP_ID config option
	 */
	public function setting_html_easify_customer_relationship_id() {
		
		$this->tooltip('6040', 
			"WooCommerce Customers will have this Customer Relationship when they are raised in Easify", 
			'easifycustomerrelationship'
		);
		
		// select control start tag
		print '<select name="easify_options_customers[easify_customer_relationship_id]" style="width: 150px;">';
		
		// get Easify customer relationships if they've not been cached
		$this->get_easify_web_service_data( 'customer_relationships' ); 
		
		// print each select option
		foreach( $this->customer_relationships as $relationship ) {
			
			// print option, mark as selected if needed
			printf (
				'<option value="%s"%s>%s</option>', 
				$relationship->Id, 
				( $this->options[ 'easify_options_customers' ][ 'easify_customer_relationship_id' ] == $relationship->Id ) || 
					( !isset( $this->options[ 'easify_options_customers' ][ 'easify_customer_relationship_id' ] ) && 
					$relationship->Description == 'Active' ) ? ' selected' : '', 
				$relationship->Description
			);
			
		}
		
		// select control close tag
		print '</select>';
		
	}
	
	
	
	/**
	 * initialise Easify Shipping page
	 */
	private function init_shipping() {
		
		// register Shipping options group 
		register_setting(
            'easify_options_shipping_group', // group
            'easify_options_shipping', // name
            array( $this, 'sanitise' ) // sanitise method
        );
		
		// add Shipping page 
		add_settings_section(
            'easify_options_section_shipping', // id
            'Easify Shipping Options', // title
            array( $this, 'settings_html_easify_shipping_message' ), // callback
            'easify_options_page_shipping' // page
        );
		
		// EASIFY_DELIVERY_SKUS
		add_settings_field(
            'easify_shipping_mapping', 
            'Shipping Mapping', 
            array( $this, 'setting_html_easify_shipping_mapping' ), 
            'easify_options_page_shipping', 
            'easify_options_section_shipping' 
        );
	}
	
	/** 
     * HTML to display for the Shipping section
     */
    public function settings_html_easify_shipping_message() {
		
        print 'These settings will be used to link Easify products to WooCommerce shipping options...';
		
    }
	
	/**
	 * HTML to display the EASIFY_SHIPPING_MAPPING config option
	 */
	public function setting_html_easify_shipping_mapping() {
		
?>
<table class="wc_shipping widefat">
	<thead>
		<tr>
			<th>Name</th>
			<th>Easify SKU</th>
		</tr>
	</thead>
	<tbody>
		<tr>
			<td>
				Flat Rate (cost per order)
			</td>
			<td>
<?php
				$this->tooltip('6050', 
					"&lt;b&gt;Easify Updates:&lt;/b&gt; The price of the Easify product represented by this SKU number&lt;br /&gt; 
					will be linked to the &lt;b&gt;cost per order&lt;/b&gt; value of the WooCommerce &lt;b&gt;Flat Rate&lt;/b&gt; shipping option.&lt;br /&gt;&lt;br /&gt;
					&lt;b&gt;WooCommerce Updates:&lt;/b&gt; When a new WooCommerce order that has Flat Rate shipping&lt;br /&gt; 
					is raised in Easify, this product will represent the cost of shipping.", 
					'flatrate'
				);
				$this->setting_html_easify_shipping_sku( 'flat_rate' );
?>
			</td>
		</tr>
		<tr>
			<td>
				Free Shipping (minimum order amount)
			</td>
			<td>
<?php
				$this->tooltip('6050', 
					"&lt;b&gt;Easify Updates:&lt;/b&gt; The price of the Easify product represented by this SKU number&lt;br /&gt; 
					will be linked to the &lt;b&gt;minimum order amount&lt;/b&gt; value of the WooCommerce&lt;br /&gt; 
					&lt;b&gt;Free Shipping&lt;/b&gt; option.&lt;br /&gt;&lt;br /&gt;
					&lt;b&gt;WooCommerce Updates:&lt;/b&gt; When a new WooCommerce order that has &lt;b&gt;Free Shipping&lt;/b&gt;&lt;br /&gt; 
					is raised in Easify, this product will be added to the order to represent the free shipping.", 
					'freeshipping'
				);
				$this->setting_html_easify_shipping_sku( 'free_shipping' );
?>
			</td>
		</tr>
		<tr>
			<td>
				International Delivery
			</td>
			<td>
<?php
				$this->tooltip('6050', 
					"&lt;b&gt;Easify Updates:&lt;/b&gt; The price of the Easify product represented by this SKU number&lt;br /&gt; 
					will be linked to the &lt;b&gt;cost&lt;/b&gt; of the WooCommerce &lt;b&gt;International Shipping&lt;/b&gt; option.&lt;br /&gt;&lt;br /&gt;
					&lt;b&gt;WooCommerce Updates:&lt;/b&gt; When a new WooCommerce order that has &lt;b&gt;International Shipping&lt;/b&gt;&lt;br /&gt; 
					is raised in Easify, this product will be added to the order to represent the cost of shipping.", 
					'internationaldelivery'
				);
				$this->setting_html_easify_shipping_sku( 'international_delivery' );
?>
			</td>
		</tr>
		<tr>
			<td>
				Local Delivery (cost per order)
			</td>
			<td>
<?php
				$this->tooltip('6050', 
					"&lt;b&gt;Easify Updates:&lt;/b&gt; The price of the Easify product represented by this SKU number&lt;br /&gt; 
					will be linked to the &lt;b&gt;cost per order&lt;/b&gt; value of the WooCommerce &lt;b&gt;Local Delivery&lt;/b&gt; shipping option.&lt;br /&gt;&lt;br /&gt;
					&lt;b&gt;WooCommerce Updates:&lt;/b&gt; When a new WooCommerce order that has &lt;b&gt;Local Delivery&lt;/b&gt;&lt;br /&gt; 
					is raised in Easify, this product will be added to the order to represent the cost of shipping.", 
					'localdelivery'
				);
				$this->setting_html_easify_shipping_sku( 'local_delivery' );
?>
			</td>
		</tr>
		<tr>
			<td>
				Default
			</td>
			<td>
<?php
				$this->tooltip('6050', 
					"&lt;b&gt;WooCommerce Updates:&lt;/b&gt; When a new WooCommerce order that has an unsupported shipping option&lt;br /&gt; 
					is raised in Easify, this product will be added to the order to represent the cost of shipping.", 
					'default'
				);
				$this->setting_html_easify_shipping_sku( 'default' );
?>
			</td>
		</tr>
	</tbody>
</table>
<?php
		
	}
	
	/**
	 * HTML to display the EASIFY_DELIVERY_SKUS config option
	 */
	public function setting_html_easify_shipping_sku( $shipping_type = 'default' ) {
		
		printf(
			'<input type="text" id="easify_shipping_mapping" name="easify_options_shipping[easify_shipping_mapping][' . $shipping_type .']" style="width: 150px;" value="%s" />',
            isset( $this->options[ 'easify_options_shipping' ][ 'easify_shipping_mapping' ][ $shipping_type ] ) ? 
				esc_attr( $this->options[ 'easify_options_shipping' ][ 'easify_shipping_mapping' ][ $shipping_type ] ) : ''
		);
		
	}
	
	
	
	/**
	 * initialise Easify Payment page
	 */
	private function init_payment() {
		
		// register Payments options group 
		register_setting(
            'easify_options_payment_group', // group
            'easify_options_payment', // name
            array( $this, 'sanitise' ) // sanitise method
        );
		
		// add Payments page 
		add_settings_section(
            'easify_options_section_payment', // id
            'Easify Payment Options', // title
            array( $this, 'settings_html_easify_payment_message' ), // callback
            'easify_options_page_payment' // page
        );
		
		// EASIFY_PAYMENT_COMMENT
		add_settings_field(
            'easify_payment_comment', // id
            'Payment Comment', // title
            array( $this, 'setting_html_easify_payment_comment' ), // callback
            'easify_options_page_payment', // page
            'easify_options_section_payment' // section
        );
		
		// EASIFY_PAYMENT_TERMS_ID
		add_settings_field(
            'easify_payment_terms_id', 
            'Payment Terms', 
            array( $this, 'setting_html_easify_payment_terms_id' ), 
            'easify_options_page_payment', 
            'easify_options_section_payment' 
        );
		
		// EASIFY_PAYMENT_MAPPING
		add_settings_field(
            'easify_payment_mapping', 
            'Payment Mapping', 
            array( $this, 'setting_html_easify_payment_mapping' ), 
            'easify_options_page_payment', 
            'easify_options_section_payment' 
        );
		
	}
	
	/** 
     * HTML to display for the Payment section
     */
    public function settings_html_easify_payment_message() {
		
        print 'These settings will be used when Payments are automatically raised in Easify...';
		
    }
	
	/**
	 * HTML to display the EASIFY_PAYMENT_MAPPING config option
	 */
	public function setting_html_easify_payment_mapping() {
		
?>
<table class="wc_gateways widefat">
	<thead>
		<tr>
			<th>Name</th>
			<th>Method</th>
			<th>Account</th>
			<th>Raise?</th>
		</tr>
	</thead>
	<tbody>
		<tr>
			<td>
				PayPal
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce PayPay payments will have this Payment Method when they are raised in Easify", 
					'method'
				);
				$this->setting_html_easify_payment_method_id( 'paypal' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce PayPay payments will have this Payment Account when they are raised in Easify", 
					'account'
				);
				$this->setting_html_easify_payment_account_id( 'paypal' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"Choose if WooCommerce PayPay payments are raised in Easify", 
					'raise'
				);
				$this->setting_html_easify_payment_raise( 'paypal' );
?>				
			</td>
		</tr>
		<tr>
			<td>
				Cash on Delivery
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce Cash on Delivery payments will have this Payment Method when they are raised in Easify", 
					'method'
				);
				$this->setting_html_easify_payment_method_id( 'cod' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce Cash on Delivery payments will have this Payment Account when they are raised in Easify", 
					'account'
				);
				$this->setting_html_easify_payment_account_id( 'cod' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"Choose if WooCommerce Cash on Delivery payments are raised in Easify", 
					'raise'
				);
				$this->setting_html_easify_payment_raise( 'cod' );
?>				
			</td>
		</tr>
		<tr>
			<td>
				Cheque Payment
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce Cheque payments will have this Payment Method when they are raised in Easify", 
					'method'
				);
				$this->setting_html_easify_payment_method_id( 'cheque' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce Cheque payments will have this Payment Account when they are raised in Easify", 
					'account'
				);
				$this->setting_html_easify_payment_account_id( 'cheque' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"Choose if WooCommerce Cheque payments are raised in Easify", 
					'raise'
				);
				$this->setting_html_easify_payment_raise( 'cheque' );
?>				
			</td>
		</tr>
		<tr>
			<td>
				Direct Bank Transfer
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce Direct Bank Transfer payments will have this Payment Method when they are raised in Easify", 
					'method'
				);
				$this->setting_html_easify_payment_method_id( 'bacs' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"WooCommerce Direct Bank Transfer payments will have this Payment Account when they are raised in Easify", 
					'account'
				);
				$this->setting_html_easify_payment_account_id( 'bacs' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"Choose if WooCommerce Direct Bank Transfer payments are raised in Easify", 
					'raise'
				);
				$this->setting_html_easify_payment_raise( 'bacs' );
?>				
			</td>
		</tr>
		<tr>
			<td>
				Default
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"Unsupported WooCommerce payments will have this Payment Method when they are raised in Easify", 
					'method'
				);
				$this->setting_html_easify_payment_method_id( 'default' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"Unsupported WooCommerce payments will have this Payment Account when they are raised in Easify", 
					'account'
				);
				$this->setting_html_easify_payment_account_id( 'default' );
?>
			</td>
			<td>
<?php
				$this->tooltip('6060', 
					"Choose if unsupported WooCommerce payments are raised in Easify", 
					'raise'
				);
				$this->setting_html_easify_payment_raise( 'default' );
?>				
			</td>
		</tr>
	</tbody>
</table>
<?php
		
	}
	
	/**
	 * HTML to display the EASIFY_PAYMENT_COMMENT config option
	 */
	public function setting_html_easify_payment_comment() {
		
		$this->tooltip('6060', 
			"WooCommerce payments will have this Payment Comment when they are raised in Easify", 
			'easifypaymentcomment'
		);
		
		printf(
			'<input type="text" id="easify_payment_comment" name="easify_options_payment[easify_payment_comment]" value="%s" />',
            isset( $this->options[ 'easify_options_payment' ][ 'easify_payment_comment' ] ) ? 
				esc_attr( $this->options[ 'easify_options_payment' ][ 'easify_payment_comment' ] ) : 'Payment via website'
		);
		
	}
	
	/**
	 * HTML to display the EASIFY_PAYMENT_TERMS_ID config option
	 */
	public function setting_html_easify_payment_terms_id() {
		
		$this->tooltip('6060', 
			"WooCommerce orders will use this Payment Term when they are raised in Easify", 
			'easifypaymentterms'
		);
		
		// select control start tag
		print '<select name="easify_options_payment[easify_payment_terms_id]" style="width: 150px;">';
		
		// get Easify payment accounts if they've not been cached
		$this->get_easify_web_service_data( 'payment_terms' ); // make sure we have payment terms in cache $payment_terms
		
		// get current setting from WordPress Easify options
		$current_value = isset( $this->options[ 'easify_options_payment' ][ 'easify_payment_terms_id' ] ) ? 
			$this->options[ 'easify_options_payment' ][ 'easify_payment_terms_id' ] : null;
		
		// if option isn't active, skip
		foreach ( $this->payment_terms as $term ) {
			
			// print option, mark as selected if needed
			printf(
				'<option value="%s"%s>%s</option>', 
				$term->Id, 
				( $current_value == $term->Id ) || 
					( $term->Description == 'Pro Forma' ) ? ' selected' : '', 
				$term->Description
			);
			
		}
		
		// select control end tag
		print '</select>';
		
	}
	
	/**
	 * HTML to display the EASIFY_PAYMENT_METHOD_ID config option
	 */
	public function setting_html_easify_payment_method_id( $payment_type = 'default' ) {
		
		// select control start tag
		print '<select name="easify_options_payment[easify_payment_mapping][' . $payment_type . '][method_id]" style="width: 150px;">';
		
		// get Easify payment methods if they've not been cached
		$this->get_easify_web_service_data( 'payment_methods' ); 
		
		// get current setting from WordPress Easify options
		$current_value = isset( $this->options[ 'easify_options_payment' ][ 'easify_payment_mapping' ][ $payment_type ][ 'method_id' ] ) ? 
			$this->options[ 'easify_options_payment' ][ 'easify_payment_mapping' ][ $payment_type ][ 'method_id' ] : null;
		
		foreach( $this->payment_methods as $method ) {
			
			// if option isn't active, skip
			if( $method->Active != 'true' ) continue;
			
			// print option, mark as selected if needed
			printf(
				'<option value="%s"%s>%s</option>', 
				$method->Id, 
				( $current_value == $method->Id ) ? ' selected' : '', 
				$method->Description
			);
			
		}
		
		// select control end tag
		print '</select>';
		
	}
	
	/**
	 * HTML to display the EASIFY_PAYMENT_ACCOUNT_ID config option
	 */
	public function setting_html_easify_payment_account_id( $payment_type = 'default' ) {
		
		// select control start tag
		print '<select name="easify_options_payment[easify_payment_mapping][' . $payment_type . '][account_id]" style="width: 150px;">';
		
		// get Easify payment accounts if they've not been cached
		$this->get_easify_web_service_data( 'payment_accounts' ); 
		
		// get current setting from WordPress Easify options
		$current_value = isset( $this->options[ 'easify_options_payment' ][ 'easify_payment_mapping' ][ $payment_type ][ 'account_id' ] ) ? 
			$this->options[ 'easify_options_payment' ][ 'easify_payment_mapping' ][ $payment_type ][ 'account_id' ] : null;
		
		foreach( $this->payment_accounts as $account ) {
			
			// if option isn't active, skip
			if( $account->Active != 'true' ) continue;
			
			// print option, mark as selected if needed
			printf(
				'<option value="%s"%s>%s</option>', 
				$account->Id, 
				( $current_value == $account->Id ) ? ' selected' : '', 
				$account->Description
			);
			
		}
		
		// select control end tag
		print '</select>';
		
	}
	
	/**
	 * HTML to display the EASIFY_PAYMENT_RAISE config option
	 */
	public function setting_html_easify_payment_raise($payment_type = 'default') {
		
		printf (
			'<input type="checkbox" name="easify_options_payment[easify_payment_mapping][' . $payment_type . '][raise]" value="true" %s/>', 
			isset( $this->options[ 'easify_options_payment' ][ 'easify_payment_mapping' ][ $payment_type ][ 'raise' ] ) &&
			$this->options[ 'easify_options_payment' ][ 'easify_payment_mapping' ][ $payment_type ][ 'raise' ] == 'true' ? 'checked="checked"' : ''
		);
		
	}
	
	
	
	/**
	 * initialise Easify Logging page
	 */
	private function init_logging() {
		
		// register Logging options group 
		register_setting(
            'easify_options_logging_group', // group
            'easify_options_logging', // name
            array( $this, 'sanitise' ) // sanitise method
        );
		
		// Logging Options
		add_settings_section(
            'easify_options_section_logging', // id
            'Easify Logging Options', // title
            array( $this, 'settings_html_easify_logging_message' ), // callback
            'easify_options_page_logging' // page
        );
		
		// EASIFY_LOGGING_ENABLED
		add_settings_field(
            'easify_logging_enabled', // id
            'Enable Logging', // title
            array( $this, 'setting_html_easify_enable_logging' ), // callback
            'easify_options_page_logging', // page
            'easify_options_section_logging' // section
        );
		
		// EASIFY_DEBUG_ENABLED
        add_settings_field(
            'easify_logging_debugging',
            'Enable Debugging',
            array( $this, 'setting_html_easify_enable_debugging' ),
            'easify_options_page_logging',
            'easify_options_section_logging'
        );
		
	}
	
    /** 
     * HTML to display for the Logging section
     */
    public function settings_html_easify_logging_message() {
		
        print 'Configure how you want Easify to report on its activities and possible errors...';
		
    }
	
    /** 
     * HTML to display the EASIFY_LOGGING_ENABLED config option
     */
    public function setting_html_easify_enable_logging() {
		
		$this->tooltip('6070', 
			"Do you wish to save logging information?&lt;br /&gt;&lt;br /&gt;
			Logging information will be saved here on your web server if this option is enabled:&lt;br /&gt;
			&lt;code&gt;" . plugin_dir_url( __FILE__ ) . "logs/easify_log.txt&lt;/code&gt;", 
			'easifylogging'
		);
		
		printf (
			'Yes <input type="radio" id="easify_logging_enabled" name="easify_options_logging[easify_logging_enabled]" value="true" %s/>', 
			isset( $this->options[ 'easify_options_logging' ][ 'easify_logging_enabled' ] ) && 
			$this->options[ 'easify_options_logging' ][ 'easify_logging_enabled' ] == 'true' ? 'checked="checked" ' : ''
		);
		
		printf (
			'No <input type="radio" id="easify_logging_enabled" name="easify_options_logging[easify_logging_enabled]" value="false" %s/>', 
			isset( $this->options[ 'easify_options_logging' ][ 'easify_logging_enabled' ] ) &&
			$this->options[ 'easify_options_logging' ][ 'easify_logging_enabled' ] == 'true' ? '' : 'checked="checked" '
		);
		
    }
	
    /** 
     * HTML to display the EASIFY_DEBUG_ENABLED config option
     */
    public function setting_html_easify_enable_debugging() {
		
		$this->tooltip('6070', 
			"Do you wish to save debugging information?&lt;br /&gt;&lt;br /&gt;
			Debugging information will be saved here on your web server if this option is enabled:&lt;br /&gt;
			&lt;code&gt;" . plugin_dir_url( __FILE__ ) . "logs/easify_debug.txt&lt;/code&gt;", 
			'easifydebugging'
		);
		
		printf(
			'Yes <input type="radio" id="easify_logging_debugging" name="easify_options_logging[easify_logging_debugging]" value="true" %s/>', 
			isset( $this->options[ 'easify_options_logging' ][ 'easify_logging_debugging' ] ) && 
			$this->options[ 'easify_options_logging' ][ 'easify_logging_debugging' ] == 'true' ? 'checked="checked" ' : ''
		);
		
		print '&nbsp;&nbsp;&nbsp;';
		
		printf(
			'No <input type="radio" id="easify_logging_debugging" name="easify_options_logging[easify_logging_debugging]" value="false" %s/>', 
			isset( $this->options[ 'easify_options_logging' ][ 'easify_logging_debugging' ] ) &&
			$this->options[ 'easify_options_logging' ][ 'easify_logging_debugging' ] == 'true' ? '' : 'checked="checked" '
		);
		
    }
	
	
	
	/**
     * sanitize each setting field as needed
     */
    public function sanitise( $input ) {
		
		return $input;
		
    }
}

if ( is_admin() )
	$wees = new WooCommerce_Easify_EASU_Settings();

?>