<?php
/*
	#####################################
										
		Easify WooCommerce Connector
		http://www.easify.co.uk/		
										
		Copyright (c) 2014 Easify Ltd	
		Released under the GPLv2
										
	#####################################
	
	*********************************
	   EASIFY WOOCOMMERCE RECEIVER	 
	*********************************
	
	The EasifyReceiver object creates, updates and deletes products from the WooCommerce database.
	
	This is a translation script that receives messages from easify_web_service.php
	Each message is translated and actioned according to the specifics of the 
	WooCommerce WordPress plugin.
	
	EasifyReceiver Methods
	----------------------
	
	~ Helpers ~
		
		WriteLog( Text )
		HandleException( Text )
		DebugTrace()
		CreateSlug( Name )
		
	~ WOOCOMM Methods ~
		
		IsExistingProduct( SKU )
		GetWOOCOMMTaxIdByEasifyTaxId( EasifyTaxId )
		AddTaxRate( EasifyTax, TaxCode, TaxRate )
		InsertCategoryIntoWOOCOMM( Name, Description )
		InsertSubCategoryIntoWOOCOMM( Name, Description, ParentId )
		UpdateDeliveryPrice( SKU, Price )
		
	~ Main Methods ~
		
		InsertProduct( Product )
		UpdateProduct( Product )
		UpdateProductInfo( ProductSKU, Picture, HTMLDescription )
		UpdateProductStockLevel( ProductSKU, StockLevel )
		DeleteProduct( ProductSKU )
	
*/

// include Easify helper functions and import WordPress libraries
include_once(dirname(__FILE__) . '/easify_functions.php'); 

class EasifyReceiver
{
	
/* 
	Helpers 
*/

	public static function WriteLog($Text)
	{
		if (get_option('easify_options_logging')['easify_logging_enabled'] != 'true') return;
		
		// write to log file in the following format: 17-12-2012 10:15:10:000000 - $Text \n
		$File = fopen($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/easify/logs/easify_log.txt', 'a');
		fwrite($File, date('d-m-y H:i:s') . substr((string)microtime(), 1, 6) . ' - ' . $Text . "\n");
		fclose($File);
	}
	
	public static function HandleException($Text)
	{
		throw new Exception($Text);
	}
	
	public static function SendEmail($Text)
	{
		// utilise php email method with WOOCOMM settings
		try {
			mail(get_option('admin_email'), 'Receive Error From ' . get_option('blogname') . ' Website', $Text, 'From:' . get_option('woocommerce_email_from_address'));
		} catch (Exception $e) {
			self::WriteLog('SendEmail Exception: ' . $e->getMessage() . '\n');
		}
	}
	
	public static function DebugTrace()
	{
		// write current stack trace to debug log file
		try {
			if (get_option('easify_options_logging')['easify_logging_debugging'] != 'true') return;
		
			ob_start(); 
			debug_print_backtrace(); 
			$Debug = ob_get_contents(); 
			ob_end_clean(); 
		
			// write to log file in the following format: 17-12-2012 10:15:10:000000 - $text \n
			$File = fopen($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/easify/logs/easify_debug.txt', 'a');
			fwrite($File, date('d-m-y H:i:s') . substr((string)microtime(), 1, 6) . "\n" . $Debug . "\n\n");
			fclose($File);
		} catch (Exception $e) {
			self::WriteLog('Receive DebugTrace Exception: ' . $e->getMessage() . "\n");
		}
	}
	
	private static function CreateSlug($Name)
	{
		// trim white spaces at beginning and end of alias and make lowercase
		$String = trim(strtolower($Name));

		// remove any duplicate whitespace, and ensure all characters are alphanumeric
		$String = preg_replace('/(\s|[^A-Za-z0-9\-])+/', '-', $String);

		// trim dashes at beginning and end of alias
		$String = trim($String, '-');

		// if we are left with an empty string, make a date with random number
		if (trim(str_replace('-', '', $String)) == '') {
			$String = date("Y-m-d-h-i-s") . mt_rand();
		}
		return $String;
	}
	
	
/* 
	WOOCOMM Methods 
*/

	public static function IsExistingProduct($SKU)
	{
		try {
			// get number of WOOCOMM products that match the Easify SKU
			global $wpdb;
			$ProductId = $wpdb->get_var($wpdb->prepare(
				"SELECT post_id FROM " . $wpdb->postmeta . " WHERE meta_key = '_sku' AND meta_value = '%s' LIMIT 1", $SKU
			));
			return is_numeric($ProductId) ? true : false;
		} catch (Exception $e) {
			self::WriteLog("IsExistingProduct Exception: " . $e->getMessage() . "\n");
		}
	}

	private static function GetWOOCOMMProductIdFromEasifySKU($SKU)
	{
		// get WOOCOMM product id from Easify SKU
		global $wpdb;
		return $wpdb->get_var($wpdb->prepare(
			"SELECT post_id FROM " . $wpdb->postmeta . " WHERE meta_key = '_sku' AND meta_value = '%s' LIMIT 1", $SKU
		));
	}

	private static function GetWOOCOMMTaxIdByEasifyTaxId($EasifyTaxId)
	{
		// match Easify tax id with an equivalent WOOCOMM tax id 
		// if not match, add Easify tax record to WOOCOMM and return WOOCOMM class name
		$TaxCode = null;
		$EasifyTax = GetEasifyTaxRates(); 
		
		// get the description and rate of the given Easify tax id
		for ($i = 0; $i < sizeof($EasifyTax); $i++)
			if ($EasifyTax[$i]->Id == $EasifyTaxId) {
				$TaxCode = trim($EasifyTax[$i]->Description);
				$TaxRate = trim($EasifyTax[$i]->TaxRate);
			}
		
		// get WOOCOMM tax id and class name by Easify tax code
		global $wpdb;
		$TaxResult = $wpdb->get_row($wpdb->prepare(
			"SELECT tax_rate_id, IFNULL(tax_rate_class, '') AS tax_rate_class FROM " . $wpdb->prefix . "woocommerce_tax_rates" . " WHERE tax_rate_name = '%s' LIMIT 1", $TaxCode
		));
		
		// return WOOCOMM tax class name if it exists, else create new and return new WOOCOMM class name
		if (!empty($TaxResult->tax_rate_id))
			return $TaxResult->tax_rate_class;
		else
			return self::AddTaxRate($EasifyTax, $TaxCode, $TaxRate);
		
		return null;
	}

	private static function AddTaxRate($EasifyTax, $TaxCode, $TaxRate)
	{
		// default tax class is represented as a blank string in WooComm
		$TaxClass = '';
		
		// check if current tax code is the Easify default
		for ($i = 0; $i < sizeof($EasifyTax); $i++) {
			$EasifyTaxDescription = trim($EasifyTax[$i]->Description);
			if ($TaxCode == $EasifyTaxDescription)
				$TaxClass = $EasifyTax[$i]->IsDefault ? '' : self::CreateSlug($EasifyTaxDescription);
		}
		
		// get list of WOOCOMM existing tax classes
		global $wpdb;
		$TaxClassOption = $wpdb->get_row($wpdb->prepare(
			"SELECT option_value FROM " . $wpdb->options . " WHERE option_name = 'woocommerce_tax_classes' LIMIT 1", ''
		));
		
		// if we find any matches, update WOOCOMM tax classes definitions 
		if(strpos($TaxClassOption->option_value, $TaxCode) === false || $TaxCode != '') {
			$NewTaxClassOptions = $TaxClassOption->option_value . "\n" . $TaxCode;
			
			$wpdb->update(
				$wpdb->options, 
				array(
					'option_value' => $NewTaxClassOptions
				), 
				array(
					'option_name' => 'woocommerce_tax_classes'
				)
			);
		}
		
		// insert tax data into WOOCOMM
		$wpdb->insert( 
			$wpdb->prefix . 'woocommerce_tax_rates', 
			array( 
				'tax_rate_country' => '', 
				'tax_rate' => $TaxRate, 
				'tax_rate_name' => $TaxCode, 
				'tax_rate_shipping' => '1', 
				'tax_rate_class' => $TaxClass
			)
		);
		
		// return class name
		return $TaxClass;
	}
	
	private static function InsertCategoryIntoWOOCOMM($Name, $Description)
	{
		$Term = term_exists($Name, 'product_cat');
		
		// if category doesn't exist, create it
		if ($Term == 0 || $Term == null) {
			$Term = wp_insert_term($Name, 'product_cat', array('description' => $Description, 'slug' => self::CreateSlug($Name)));
			$CategoryId = $Term['term_id'];
		} else
			$CategoryId = $Term['term_id'];
		
		return $CategoryId;
	}

	private static function InsertSubCategoryIntoWOOCOMM($Name, $Description, $ParentId)
	{
		$Term = term_exists($Name, 'product_cat', $ParentId);
		
		// if subcategory doesn't exist, create it
		if ($Term == 0 || $Term == null) {
			$Term = wp_insert_term($Name, 'product_cat', array('description' => $Description, 'slug' => self::CreateSlug($Name), 'parent' => $ParentId));			
			$SubCategoryId = $Term['term_id'];
		} else
			$SubCategoryId = $Term['term_id'];
		
		return $SubCategoryId;
	}
	
	private static function UpdateDeliveryPrice($SKU, $Price)
	{
		// get WOOCOMM Easify options from WordPress database
		$EasifyOptionsShipping = get_option('easify_options_shipping');
		
		// check each supported type of shipping to see if the SKU matches any that have been mapped in the WordPress Easify options
		if ($EasifyOptionsShipping['easify_shipping_mapping']['free_shipping'] == $SKU) 
		{
			// update the minimum amount to qualify for free shipping
			$WoocommSetting = get_option('woocommerce_free_shipping_settings');
			$WoocommSetting['min_amount'] = $Price;
			update_option('woocommerce_free_shipping_settings', $WoocommSetting);
			
			return true;
		} 
		
		if ($EasifyOptionsShipping['easify_shipping_mapping']['local_delivery'] == $SKU) 
		{
			// update the local delivery flat rate fee
			$WoocommSetting = get_option('woocommerce_local_delivery_settings');
			$WoocommSetting['fee'] = $Price;
			update_option('woocommerce_local_delivery_settings', $WoocommSetting);
			
			return true;
		} 
		
		if ($EasifyOptionsShipping['easify_shipping_mapping']['flat_rate'] == $SKU) 
		{
			// update the flat rate delivery fee
			$WoocommSetting = get_option('woocommerce_flat_rate_settings');
			$WoocommSetting['cost_per_order'] = $Price;
			update_option('woocommerce_flat_rate_settings', $WoocommSetting);
			
			return true;
		} 
		
		if ($EasifyOptionsShipping['easify_shipping_mapping']['international_delivery'] == $SKU) 
		{
			// update the cost of international delivery
			$WoocommSetting = get_option('woocommerce_international_delivery_settings');
			$WoocommSetting['cost'] = $Price;
			update_option('woocommerce_international_delivery_settings', $WoocommSetting);
			
			return true;
		}
		
		// we're not dealing with shipping here, so tell the calling method to continue...
		return false;
	}
	
	
	
/*
	Main Methods
*/
	
	public static function InsertProduct($Product)
	{
		try {
			// calculate price from retail margin and cost price
			$Price = round(($Product->CostPrice / (100 - $Product->RetailMargin) * 100), 4);
			
			// catch reserved delivery SKUs and update delivery prices
			if (self::UpdateDeliveryPrice($Product->SKU, $Price)) return;
			
			// sanitise weight value
			$Product->Weight = (isset($Product->Weight) && is_numeric($Product->Weight) ? $Product->Weight : 0);
			
			// get Easify product categories
			$EasifyCategories = GetEasifyProductCategories();
			
			// get Easify category description by the Easify category id
			$CategoryDescription = GetEasifyCategoryDescriptionFromEasifyCategoryId($EasifyCategories, $Product->CategoryId);
			
			// get Easify product sub categories by Easify category id
			$EasifySubCategories = GetEasifyProductSubCategoriesByCategory($Product->CategoryId);
			
			// get Easify sub category description by Easify sub category id
			$SubCategoryDescription = GetEasifyCategoryDescriptionFromEasifyCategoryId($EasifySubCategories, $Product->SubcategoryId);
			
			// insert new category if needed and return WOOCOMM category id
			$CategoryId = self::InsertCategoryIntoWOOCOMM($CategoryDescription, $CategoryDescription);
			
			// insert new sub category if needed and return WOOCOMM sub category id
			$SubCategoryId = self::InsertSubCategoryIntoWOOCOMM($SubCategoryDescription, $SubCategoryDescription, $CategoryId);
			
			// create a WOOCOMM stub for the new product
			$ProductStub = array(
				'post_title' => $Product->Description,
				'post_content' => '',
				'post_status' => 'publish',
				'post_type' => 'product'
			);
			
			// insert product record and get WOOCOMM product id
			$ProductId = wp_insert_post($ProductStub);
			
			// link subcategory to product
			wp_set_post_terms($ProductId, array($SubCategoryId), "product_cat");
			
			// get WOOCOMM tax class from Easify tax id
			$TaxClass = self::GetWOOCOMMTaxIdByEasifyTaxId($Product->TaxId);
			
			/*
				flesh out product record meta data
			*/
			
			// pricing
			update_post_meta($ProductId, '_sku', $Product->SKU);
			update_post_meta($ProductId, '_price', $Price);
			update_post_meta($ProductId, '_regular_price', $Price);
			update_post_meta($ProductId, '_sale_price', $Price);
			update_post_meta($ProductId, '_sale_price_dates_from	', '');
			update_post_meta($ProductId, '_sale_price_dates_to', '');
			update_post_meta($ProductId, '_tax_status', 'taxable');
			update_post_meta($ProductId, '_tax_class', $TaxClass);
			
			// handling stock
			update_post_meta($ProductId, '_stock_status', 'instock');
			update_post_meta($ProductId, '_manage_stock', 'yes');
			update_post_meta($ProductId, '_downloadable', 'no');
			update_post_meta($ProductId, '_virtual', 'no');
			update_post_meta($ProductId, '_visibility', 'visible');
			update_post_meta($ProductId, '_sold_individually', '');
			update_post_meta($ProductId, '_manage_stock', 'yes');
			update_post_meta($ProductId, '_backorders', 'no');
			update_post_meta($ProductId, '_stock', $Product->StockLevel);
			
			// physical properties
			update_post_meta($ProductId, '_weight', $Product->Weight);
			update_post_meta($ProductId, '_length', '');
			update_post_meta($ProductId, '_width', '');
			update_post_meta($ProductId, '_height', '');
			
			// misc
			update_post_meta($ProductId, '_purchase_note', '');
			update_post_meta($ProductId, '_featured', 'no');
			update_post_meta($ProductId, '_product_attributes', 'a:0:{}');	// no attributes
			
		} catch (Exception $e) {
			self::HandleException("InsertProductIntoDatabase Exception: " . $e->getMessage() . "\n");
		}
	}

	public static function UpdateProduct($Product)
	{		
		try {
			// calculate price from retail margin and cost price
			$Price = round(($Product->CostPrice / (100 - $Product->RetailMargin) * 100), 4);
			
			// catch reserved delivery SKUs and update delivery prices
			if (self::UpdateDeliveryPrice($Product->SKU, $Price)) return;
			
			// sanitise weight value
			$Product->Weight = (isset($Product->Weight) && is_numeric($Product->Weight) ? $Product->Weight : 0);
			
			// get Easify product categories
			$EasifyCategories = GetEasifyProductCategories();
			
			// get Easify category description by the Easify category id
			$CategoryDescription = GetEasifyCategoryDescriptionFromEasifyCategoryId($EasifyCategories, $Product->CategoryId);
			
			// get Easify product sub categories by Easify category id
			$EasifySubCategories = GetEasifyProductSubCategoriesByCategory($Product->CategoryId);
			
			// get Easify sub category description by Easify sub category id
			$SubCategoryDescription = GetEasifyCategoryDescriptionFromEasifyCategoryId($EasifySubCategories, $Product->SubcategoryId);
			
			// insert new category if needed and return WOOCOMM category id
			$CategoryId = self::InsertCategoryIntoWOOCOMM($CategoryDescription, $CategoryDescription);
			
			// insert new sub category if needed and return WOOCOMM sub category id
			$SubCategoryId = self::InsertSubCategoryIntoWOOCOMM($SubCategoryDescription, $SubCategoryDescription, $CategoryId);
			
			// get WOOCOMM product id from Easify SKU
			$ProductId = self::GetWOOCOMMProductIdFromEasifySKU($Product->SKU);
			
			// create a WOOCOMM stub for the new product
			$ProductStub = array(
				'ID' => $ProductId, 
				'post_title' => $Product->Description,
				'post_content' => '',
				'post_status' => 'publish',
				'post_type' => 'product'
			);
			
			// insert product record and get WOOCOMM product id
			$ProductId = wp_update_post($ProductStub);
			
			// link subcategory to product
			wp_set_post_terms($ProductId, array($SubCategoryId), "product_cat");
			
			// get WOOCOMM tax class from Easify tax id
			$TaxClass = self::GetWOOCOMMTaxIdByEasifyTaxId($Product->TaxId);
			
			/*
				flesh out product record meta data
			*/
			
			// pricing
			update_post_meta($ProductId, '_sku', $Product->SKU);
			update_post_meta($ProductId, '_price', $Price);
			update_post_meta($ProductId, '_regular_price', $Price);
			update_post_meta($ProductId, '_sale_price', $Price);
			update_post_meta($ProductId, '_sale_price_dates_from	', '');
			update_post_meta($ProductId, '_sale_price_dates_to', '');
			update_post_meta($ProductId, '_tax_status', 'taxable');
			update_post_meta($ProductId, '_tax_class', $TaxClass);
			
			// handling stock
			update_post_meta($ProductId, '_stock_status', 'instock');
			update_post_meta($ProductId, '_manage_stock', 'yes');
			update_post_meta($ProductId, '_downloadable', 'no');
			update_post_meta($ProductId, '_virtual', 'no');
			update_post_meta($ProductId, '_visibility', 'visible');
			update_post_meta($ProductId, '_sold_individually', '');
			update_post_meta($ProductId, '_manage_stock', 'yes');
			update_post_meta($ProductId, '_backorders', 'no');
			update_post_meta($ProductId, '_stock', $Product->StockLevel);
			
			// physical properties
			update_post_meta($ProductId, '_weight', $Product->Weight);
			update_post_meta($ProductId, '_length', '');
			update_post_meta($ProductId, '_width', '');
			update_post_meta($ProductId, '_height', '');
			
			// misc
			update_post_meta($ProductId, '_purchase_note', '');
			update_post_meta($ProductId, '_featured', 'no');
			update_post_meta($ProductId, '_product_attributes', 'a:0:{}');	// no attributes
			
		} catch (Exception $e) {
			self::HandleException("UpdateProductInDatabase Exception: " . $e->getMessage() . "\n");
		}
	}
	
	public static function UpdateProductInfo($ProductSKU, $Picture, $HTMLDescription)
	{
		try {
			// get WordPress current image upload path
			$ImageDirectory = wp_upload_dir()['path'] . '/';
			
			// save directory on the web server if required
			if (!is_dir($ImageDirectory))
				mkdir($ImageDirectory, 0777, true);
			
			// create image file name and save image byte array
			$FileName = $ImageDirectory . $ProductSKU . ".jpg"; 
			$ByteArray = $Picture; 
			$fp = fopen($FileName, "w");
			fwrite($fp, $ByteArray); 
			fclose($fp); 
			
			// get WOOCOMM product id from Easify SKU
			$ProductId = self::GetWooCommProductIdFromEasifySKU($ProductSKU);
			
			// get the file type of the image we've just uploaded
			$FileType = wp_check_filetype(basename($FileName), null);

			// get WordPress current image upload URL
			$UploadFolder = wp_upload_dir()['url'] . '/';
			
			// create a WOOCOMM stub for product image / post attachment
			$Attachment = array(
				'guid'           => $UploadFolder . basename($FileName), 
				'post_mime_type' => $FileType['type'], 
				'post_title'     => preg_replace( '/\.[^.]+$/', '', basename($FileName)), 
				'post_content'   => '', 
				'post_status'    => 'inherit' 
			);
			
			// insert product image and get WOOCOMM attachment id
			$AttachmentId = wp_insert_attachment($Attachment, $FileName, $ProductId);
			
			// generate the meta data for the attachment, and update the database record.
			$AttachData = wp_generate_attachment_metadata($AttachmentId, $FileName);
			
			// insert product image meta data 
			wp_update_attachment_metadata($AttachmentId, $AttachData);
			
			// link product record and product image thumbnail
			update_post_meta($ProductId, '_thumbnail_id', $AttachmentId);
			
		} catch (Exception $e) {
			self::HandleException("UpdateProductInfoInDatabase Exception: " . $e->getMessage() . "\n");
		}
	}
	
	public static function UpdateProductStockLevel($ProductSKU, $StockLevel)
	{		
		// get WOOCOMM product id from Easify SKU
		$ProductId = self::GetWooCommProductIdFromEasifySKU($ProductSKU);
		
		// update the stock level of the given product by it's SKU number
		global $wpdb;
		return $wpdb->update(
			$wpdb->postmeta, 
			array(
				'meta_value' => $StockLevel
			),
			array(
				'post_id' => $ProductId, 
				'meta_key' => '_stock'
			)
		);
	}
	
	public static function DeleteProduct($ProductSKU)
	{
		// get the WOOCOMM product id by the Easify SKU
		$ProductId = self::GetWooCommProductIdFromEasifySKU($ProductSKU);
		
		// also deleted: comments, post meta fields, and relationships between the post and taxonomy terms
		wp_delete_post($ProductId, true);
	}
	
}

// log all related php errors?
if (get_option('easify_options_logging')['easify_logging_enabled'] == 'true') 
	set_error_handler('php_error_handler');
function php_error_handler($errno, $errstr, $errfile, $errline) 
{
	$enum = '';
	// translate php error number
	switch($errno) {
		case E_ERROR:
			$enum = 'E_ERROR';
		break;
		case E_WARNING:
			$enum = 'E_WARNING';
		break;
		case E_PARSE:	
			$enum = 'E_PARSE';
		break;
		case E_NOTICE:
			$enum = 'E_NOTICE';
		break;
		case E_CORE_ERROR:
			$enum = 'E_CORE_ERROR';
		break;
		case E_CORE_WARNING:
			$enum = 'E_CORE_WARNING';
		break;
		case E_COMPILE_ERROR:
			$enum = 'E_COMPILE_ERROR';
		break;
		case E_COMPILE_WARNING:
			$enum = 'E_COMPILE_WARNING';
		break;
		case E_USER_ERROR:
			$enum = 'E_USER_ERROR';
		break;
		case E_USER_WARNING:
			$enum = 'E_USER_WARNING';
		break;
		case E_USER_NOTICE:
			$enum = 'E_USER_NOTICE';
		break;
		case E_STRICT:
			$enum = 'E_STRICT';
		break;
		case 4096:
			$enum = 'E_RECOVERABLE_ERROR';
		break;
		case E_DEPRECATED:
			$enum = 'E_DEPRECATED';
		break;
		case E_USER_DEPRECATED:
			$enum = 'E_USER_DEPRECATED';
		break;
		case E_ALL:
			$enum = 'E_ALL';
		break;
		default:
			$enum = $errno;
		break;
	}
	
	// handle the php error in cart module code, log to text file and or email to tech support
	EasifyReceiver::HandleException($enum . 
		(!empty($errfile) ? " [" . $errfile . "] " . 
		(!empty($errline) ? "-> [Line: " . $errline . "]" : "") : "") . 
		" - " . $errstr);
}

?>