<?php
/*
	#####################################
										
		Easify WooCommerce Connector
		http://www.easify.co.uk/		
										
		Copyright (c) 2014 Easify Ltd	
		Released under the GPLv2
										
	#####################################

	*********************************
		EASIFY WOOCOMMERCE SENDER	 
	*********************************
	
	This is a translation script that sends messages to the Easify Advanced Server Upgrade (EASU) 
	web services. The order number of a particular order is given to this script, then all 
	required information is compiled and sent to the EASU web services.
	
	EasifySender Methods
	--------------------
	
	~ Member Variables ~
		
		EasifyCustomer
		EasifyOrder
		EasifyOrderItems
		EasifyPayment
		EasifyOrderNotes
		
	~ Constructor ~
		
		__constructor( OrderNo )
		
	~ Helpers ~ 
		
		WriteLog( Text )
		HandleException( Text )
		SendEmail( Text )
		DebugTrace()
		
	~ Getters ~
		
		GetDate()
		GetTaxRateDefault()
		GetProductsEasifyTaxId( EasifyTaxRates, Products )
		GetProductsEasifyTaxRate( EasifyTaxRates, Products )
		GetEasifySKUByWooCommProductId( ProductId )
		
	~ Setters ~
		
		SetCustomer( CustomerId, OrderPostMeta )	
		SetOrder( OrderNo, CustomerNo, Paid )
		SetOrderItem( OrderNo, Product, TaxId, TaxRate )	
		SetOrderNote( OrderNo, Order )
		SetOrderShippingItem( OrderNo, Delivery )
		SetPayment( OrderNo, OrderPostMeta )
	
	~ EASU Web Service Invocation ~
		
		ThrowStuffToEasify()
*/

// include Easify helper functions and import WordPress libraries
include_once(dirname(__FILE__) . '/easify_functions.php'); 

class EasifySender 
{

/* 
	Member Variables 
*/

	private $EasifyCustomer, $EasifyOrder, $EasifyOrderItems, $EasifyPayment, $EasifyOrderNotes;
	
	
	
/* 
	Constructor 
*/

	function __construct($OrderNo)
	{
		// compile all required information and send it to EASU web services
		
		try {
			
			self::WriteLog("In constructor - order no:" . $OrderNo);
			
			// get order data.. containing customer id, products data, shipping data and notes data
			$Order = new WC_Order($OrderNo);
			
			// get order meta data.. containing customer data and payment data
			$OrderPostMeta = get_post_meta($OrderNo);
			
			// get customer id 
			$CustomerId = (int)$Order->user_id;
			
			self::WriteLog("In constructor - cust no:" . $CustomerId);
			
			 // set customer data
			$this->SetCustomer($CustomerId, $OrderPostMeta);
			
			self::WriteLog("In constructor - customer set.");
			
			// get order products data
			$OrderDetails = $Order->get_items();
			
			// get payment method option, do we raise payment in Easify?
			$RaisePayment = (
				isset(
					get_option('easify_options_payment')
					['easify_payment_mapping']
					[$OrderPostMeta['_payment_method'][0]]
					['raise']
				) && 
				get_option('easify_options_payment')
				['easify_payment_mapping']
				[$OrderPostMeta['_payment_method'][0]]
				['raise'] == 'true' ? true : false
			);
			
			// set order data
			$this->SetOrder($OrderNo, $CustomerId, $RaisePayment);
			
			self::WriteLog("In constructor - order set.");
			
			// get tax rates data
			$EasifyTaxRates = GetEasifyTaxRates();
			
			// set order details data 
			foreach ($OrderDetails as $Product) {
				// translate WooComm tax into Easify tax 
				$TaxId = $this->GetProductsEasifyTaxId($EasifyTaxRates, $Product);
				$TaxRate = $this->GetProductsEasifyTaxRate($EasifyTaxRates, $Product);
				
				// set current order detail data
				$this->SetOrderItem($OrderNo, $Product, $TaxId, $TaxRate);
			}
			
			self::WriteLog("In constructor - order details set.");
			
			// get order shipping data 
			$ShippingItems = $Order->get_shipping_methods();
			
			// set order delivery items
			foreach ($ShippingItems as $ShippingItem)
				$this->SetOrderShippingItem($OrderNo, $ShippingItem);
			
			self::WriteLog("In constructor - delivery set.");
			
			// set payment data
			if ($RaisePayment) 
				$this->SetPayment($OrderNo, $OrderPostMeta);
			
			self::WriteLog("In constructor - payment set.");
			
			// set internal notes
			$this->SetOrderNote($OrderNo, $Order);
			
			self::WriteLog("In constructor - order notes set.");
			
			// send package to the easify server 
			if (!$this->ThrowStuffToEasify())
				self::SendEmail("Order (" . $OrderNo . ") failed to submit to Easify.");
			
		} catch(Exception $ex) {
			self::WriteLog("__constructor() Exception: " . $ex->getMessage());
			self::HandleException("__constructor() Exception: " . $ex->getMessage());
			return false;
		}
   	}
	
	
	
/* 
	Helpers 
*/
	
	public static function WriteLog($Text)
	{
		if (get_option('easify_options_logging')['easify_logging_enabled'] != 'true') return;
		
		// write to log file in the following format: 17-12-2012 10:15:10:000000 - $Text \n
		$File = fopen($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/easify/logs/easify_log.txt', 'a');
		fwrite($File, date('d-m-y H:i:s') . substr((string)microtime(), 1, 6) . ' - ' . $Text . " \n");
		fclose($File);
	}
	
	public static function HandleException($Text)
	{
		throw new Exception($Text);
	}
	
	public static function SendEmail($Text)
	{
		// utilise php email method with WOOCOMM settings
		try {
			mail(get_option('admin_email'), 'Send Error From ' . get_option('blogname') . ' Website', $Text, 'From:' . get_option('woocommerce_email_from_address'));
		} catch (Exception $e) {
			self::WriteLog('SendEmail Exception: ' . $e->getMessage() . "\n");
		}
	}
	
	public static function DebugTrace()
	{
		// write current stack trace to debug log file
		try {
			if (get_option('easify_options_logging')['easify_logging_debugging'] != 'true') return;
		
			ob_start(); 
			debug_print_backtrace(); 
			$Debug = ob_get_contents(); 
			ob_end_clean(); 
		
			// write to log file in the following format: 17-12-2012 10:15:10:000000 - $text \n
			$File = fopen($_SERVER['DOCUMENT_ROOT'] . '/wp-content/plugins/easify/logs/easify_debug.txt', 'a');
			fwrite($File, date('d-m-y H:i:s') . substr((string)microtime(), 1, 6) . "\n" . $Debug . "\n\n");
			fclose($File);
		} catch (Exception $e) {
			self::WriteLog('Send DebugTrace Exception: ' . $e->getMessage() . "\n");
		}
	}
	
	
	
/*	
	Getters 
*/

	private static function GetDate() {
		return date('Y-m-d\Th:i:s', time());
	}
	
	private static function GetTaxRateDefault() {
		$tax = GetEasifyTaxRates();
		for ($x = 0; $x < sizeof($tax) - 1; $x++)
			if ($tax[$x]->IsDefault)
				return $tax[$x];
	}
	
	function GetProductsEasifyTaxId($EasifyTaxRates, $Products) {
		
		$Tax = new WC_Tax();
		$TaxRates = $Tax->get_rates($Products['tax_class']);
		$Rates = array_shift($TaxRates);
		
		// match Easify tax rate to WOOCOMM tax rate and return Easify tax id
		if (isset($Rates['rate']))
			for ($x = 0; $x <= sizeof($EasifyTaxRates); $x++)
				if ($Rates['rate'] == $EasifyTaxRates[$x]->TaxRate)
					return $EasifyTaxRates[$x]->Id;
		
		// get default Easify tax id
		for ($x = 0; $x < sizeof($EasifyTaxRates) - 1; $x++)
			if ($EasifyTaxRates[$x]->IsDefault)
				return $EasifyTaxRates[$x]->Id;
				
		// if no default found, presume 4, Easify's default tax id for zero rated
		return 4;
	}
	
	function GetProductsEasifyTaxRate($EasifyTaxRates, $Products) {
		
		$Tax = new WC_Tax();
		$TaxRates = $Tax->get_rates($Products['tax_class']);
		$Rates = array_shift($TaxRates);
		
		// match Easify tax rate to WOOCOMM  tax rate and return Easify tax rate
		if (isset($Rates['rate'])) {
			for ($x = 0; $x <= sizeof($EasifyTaxRates); $x++) {
				if ($Rates['rate'] == $EasifyTaxRates[$x]->TaxRate) {
					return $EasifyTaxRates[$x]->TaxRate;
				}
			}
		}
		
		// get default Easify tax id
		for ($x = 0; $x < sizeof($EasifyTaxRates) - 1; $x++)
			if ($EasifyTaxRates[$x]->IsDefault)
				return $EasifyTaxRates[$x]->TaxRate;
				
		// if no default found, presume zero rated
		return 0;
	}
	
	function GetEasifySKUByWooCommProductId($ProductId) {
		// get the Easify SKU relating to a WOOCOMM product id
		global $wpdb;
		return $wpdb->get_var($wpdb->prepare(
			"SELECT meta_value FROM " . $wpdb->postmeta . " WHERE meta_key = '_sku' AND post_id = '%s' LIMIT 1", $ProductId
		));
	}
	
	
	
/*
	Setters
*/
	
	private function SetCustomer($CustomerId, $OrderPostMeta)
	{
		// parse WOOCOMM customer array into Easify customer array
		$this->EasifyCustomer = array("ExtCustomerId" => $CustomerId,
			"CompanyName" => $OrderPostMeta['_billing_company'][0], 
			"Title" => "",
			"FirstName" => $OrderPostMeta['_billing_first_name'][0], 
			"Surname" => $OrderPostMeta['_billing_last_name'][0], 
			"JobTitle" => "",
			"Address1" => $OrderPostMeta['_billing_address_1'][0], 
			"Address2" => $OrderPostMeta['_billing_address_2'][0], 
			"Address3" => "", 
			"Town" => $OrderPostMeta['_billing_city'][0], 
			"County" => $OrderPostMeta['_billing_state'][0],
			"Postcode" => $OrderPostMeta['_billing_postcode'][0], 
			"Country" => $OrderPostMeta['_billing_country'][0], 
			"HomeTel" => $OrderPostMeta['_billing_phone'][0], 
			"Email" => $OrderPostMeta['_billing_email'][0], 
			"DeliveryFirstName" => $OrderPostMeta['_shipping_first_name'][0], 
			"DeliverySurname" => $OrderPostMeta['_shipping_last_name'][0], 
			"DeliveryCompanyName" => $OrderPostMeta['_shipping_company'][0], 
			"DeliveryAddress1" => $OrderPostMeta['_shipping_address_1'][0], 
			"DeliveryAddress2" => $OrderPostMeta['_shipping_address_2'][0], 
			"DeliveryAddress3" => "", 
			"DeliveryTown" => $OrderPostMeta['_shipping_city'][0], 
			"DeliveryCounty" => $OrderPostMeta['_shipping_state'][0], 
			"DeliveryPostcode" => $OrderPostMeta['_shipping_postcode'][0], 
			"DeliveryCountry" => $OrderPostMeta['_shipping_country'][0], 
			"DeliveryTel" => "", 
			"DeliveryEmail" => "", 
			"SubscribeToNewsletter" => "",
			"TradeAccount" => false,
			"CreditLimit" => 0,
			"CustomerTypeId" => get_option('easify_options_customers')['easify_customer_type_id'],
			"PaymentTermsId" => get_option('easify_options_payment')['easify_payment_terms_id'],
			"RelationshipId" => get_option('easify_options_customers')['easify_customer_relationship_id'],
			"PK" => get_option('easify_options_setup')['easify_private_key']);
	}
	
	private function SetOrder($OrderNo, $CustomerNo, $Paid)
	{
		// parse WOOCOMM order array into Easify order array
		$this->EasifyOrder = array("ExtOrderNo" => $OrderNo, 
		"ExtCustomerId" => $CustomerNo, 
		"DatePlaced" => self::GetDate(), 
		"StatusId" => get_option('easify_options_orders')['easify_order_status_id'], 
		"Paid" => $Paid, 
		"CustomerRef" => '',
		"Invoiced" => true, 
		"DateInvoiced" => self::GetDate(), 
		"Comments" => get_option('easify_options_orders')['easify_order_comment'] . " " . $OrderNo, 
		"DateOrdered" => self::GetDate(), 
		"DatePaid" => self::GetDate(), 
		"DueDate" => self::GetDate(), 
		"DueTime" => self::GetDate(), 
		"Scheduled" => false, 
		"Duration" => 0, 
		"Priority" => 0,
		"Recurring" => false, 
		"RecurTimePeriod" => 0, 
		"UseTradeMargins" => false, 
		"DueDate2" => self::GetDate(), 
		"DueTime2" => self::GetDate(), 
		"DueDuration2" => 0,
		"OrderType" => get_option('easify_options_orders')['easify_order_type_id'], 
		"PaymentTermsId" => get_option('easify_options_payment')['easify_payment_terms_id']);
	}
	
	private function SetOrderItem($OrderNo, $Product, $TaxId, $TaxRate)
	{ 
		// parse WOOCOMM order product array into Easify order product array
		$this->EasifyOrderItems[sizeof($this->EasifyOrderItems)] = array("Sku" => $this->GetEasifySKUByWooCommProductId($Product['product_id']),
		"Qty" => $Product['qty'],
		"Price" => round($Product['line_subtotal'] / ($Product['qty'] == 0 ? 1 : $Product['qty']), 4),
		"Comments" => '',
		"TaxRate" => $TaxRate,
		"TaxId" => $TaxId,
		"Spare" => '',
		"ExtParentId" => 0,
		"ExtOrderDetailsId" => $Product['product_id'],
		"ExtOrderNo" => $OrderNo,
		"AutoAllocateStock" => true);
	}
	
	private function SetOrderNote($OrderNo, $Order)
	{
		// parse notes given by customer during WOOCOMM checkout into Easify order notes
		$this->EasifyOrderNotes[0] = array("ExtOrderNo" => $OrderNo, "Notes" => $Order->customer_note . "\n\n");
	}
	
	private function SetOrderShippingItem($OrderNo, $ShippingItem)
	{
		// all shipping items are raised in Easify at the default tax rate set in Easify
		$TaxDefault = $this->GetTaxRateDefault();
		
		// map current shipping item to an Easify product 
		foreach (get_option('easify_options_shipping')['easify_shipping_mapping'] as $Name => $SKU) {
			if ($Name == $ShippingItem['method_id']) {
				// parse WOOCOMM shipping item into Easify order products array 
				$this->EasifyOrderItems[sizeof($this->EasifyOrderItems)] = array("Sku" => $SKU, 
					"Qty" => 1, 
					"Price" => $ShippingItem['cost'], 
					"Comments" => $ShippingItem['name'], // record in line item 
					"TaxRate" => $TaxDefault->TaxRate, 
					"TaxId" => $TaxDefault->Id, 
					"Spare" => "", 
					"ExtParentId" => 0, 
					"ExtOrderDetailsId" => 1, 
					"ExtOrderNo" => $OrderNo, 
					"AutoAllocateStock" => true);
			}
		}
	}
	
	private function SetPayment($OrderNo, $OrderPostMeta)
	{
		// determine which Easify payment method id/account id we should use...
		$PaymentMappings = get_option('easify_options_payment')['easify_payment_mapping'][$OrderPostMeta['_payment_method'][0]];
		if (!isset($PaymentMappings))
			$PaymentMappings = get_option('easify_options_payment')['easify_payment_mapping']['default'];
		
		// setup defaults in case we don't find a match
		$PaymentMethodId = $PaymentMappings['method_id'];
		$PaymentAccountId = $PaymentMappings['account_id'];
		
		// parse WOOCOMM payment array into Easify payment array
		$this->EasifyPayment = array("PaymentDate" => self::GetDate(), 
			"PaymentAccountId" => $PaymentAccountId, 
			"TransactionRef" => $OrderPostMeta['_payment_method_title'][0], 
			"PaymentMethodId" => $PaymentMethodId, 
			"PaymentTypeId" => 1, // should always be a sale type, default(1)
			"ExtOrderNo" => $OrderNo, 
			"Comments" => get_option('easify_options_payment')['easify_payment_comment'], 
			"Amount" => $OrderPostMeta['_order_total'][0]);
	}
	
	
	
/*
	EASU Web Service Invocation
*/

	private function ThrowStuffToEasify()
	{
		
		self::WriteLog("ThrowStuffToEasify() - Sending customer to Easify");
		
		try {
			InsertEasifyCustomer($this->EasifyCustomer);
		} catch(Exception $ex) {
			self::HandleException("ThrowStuffToEasify() Exception: " . $ex->getMessage() . "\n" . "Customer: " . print_r($this->EasifyCustomer, true));
			return false;
		}
		
		self::WriteLog("ThrowStuffToEasify() - Sending order to Easify");
		
		try {
			InsertEasifyOrder($this->EasifyOrder);
		} catch(Exception $ex) {
			self::HandleException("ThrowStuffToEasify() Exception: " . $ex->getMessage() . "\n" . "Order: " . print_r($this->EasifyOrder, true));
			return false;
		}
		
		self::WriteLog("ThrowStuffToEasify() - Sending order details to Easify - item count:" . sizeof($this->EasifyOrderItems));
		
		try {
			InsertEasifyOrderItems($this->EasifyOrderItems);
		} catch(Exception $ex) {
			self::HandleException("ThrowStuffToEasify() Exception: " . $ex->getMessage() . "\n" . "OrderDetails: " . print_r($this->EasifyOrderItems, true));
			return false;
		}
		
		self::WriteLog("ThrowStuffToEasify() - Sending payment to Easify");
		
		try {
			if(isset($this->EasifyPayment) && !empty($this->EasifyPayment))
				InsertEasifyPayment($this->EasifyPayment);
		} catch(Exception $ex) {
			self::HandleException("ThrowStuffToEasify() Exception: " . $ex->getMessage() . "\n" . "Payment: " . print_r($this->EasifyPayment, true));
			return false;
		}
		
		self::WriteLog("ThrowStuffToEasify() - Sending order notes to Easify - item count:" . sizeof($this->EasifyOrderNotes));
		
		try {
			InsertEasifyInternalNotes($this->EasifyOrderNotes);
		} catch(Exception $ex) {
			self::HandleException("ThrowStuffToEasify() Exception: " . $ex->getMessage() . "\n" . "Order Notes: " . print_r($this->EasifyOrderNotes[$i], true));
			return false;
		}
		
		self::WriteLog("ThrowStuffToEasify() - End");
		return true;
	}
}

// log all related php errors if logging enabled
if (get_option('easify_options_logging')['easify_logging_enabled'] == 'true') 
	set_error_handler('php_error_handler');
function php_error_handler($errno, $errstr, $errfile, $errline) 
{
	$enum = '';
	// translate php error number
	switch($errno) {
		case E_ERROR:
			$enum = 'E_ERROR';
		break;
		case E_WARNING:
			$enum = 'E_WARNING';
		break;
		case E_PARSE:	
			$enum = 'E_PARSE';
		break;
		case E_NOTICE:
			$enum = 'E_NOTICE';
		break;
		case E_CORE_ERROR:
			$enum = 'E_CORE_ERROR';
		break;
		case E_CORE_WARNING:
			$enum = 'E_CORE_WARNING';
		break;
		case E_COMPILE_ERROR:
			$enum = 'E_COMPILE_ERROR';
		break;
		case E_COMPILE_WARNING:
			$enum = 'E_COMPILE_WARNING';
		break;
		case E_USER_ERROR:
			$enum = 'E_USER_ERROR';
		break;
		case E_USER_WARNING:
			$enum = 'E_USER_WARNING';
		break;
		case E_USER_NOTICE:
			$enum = 'E_USER_NOTICE';
		break;
		case E_STRICT:
			$enum = 'E_STRICT';
		break;
		case 4096:
			$enum = 'E_RECOVERABLE_ERROR';
		break;
		case E_DEPRECATED:
			$enum = 'E_DEPRECATED';
		break;
		case E_USER_DEPRECATED:
			$enum = 'E_USER_DEPRECATED';
		break;
		case E_ALL:
			$enum = 'E_ALL';
		break;
		default:
			$enum = $errno;
		break;
	}
	
	// handle the php error and log it
	EasifySender::WriteLog($enum . 
		(!empty($errfile) ? " [" . $errfile . "] " . 
		(!empty($errline) ? "-> [Line: " . $errline . "]" : "") : "") . 
		" - " . $errstr);
}

?>